# Generated by roxygen2: do not edit by hand

S3method(augment,matching_result)
S3method(balance_diagnostics,cem_result)
S3method(balance_diagnostics,default)
S3method(balance_diagnostics,full_matching_result)
S3method(balance_diagnostics,matching_result)
S3method(balance_diagnostics,subclass_result)
S3method(dim,lazy_cost_spec)
S3method(ggplot2::autoplot,balance_diagnostics)
S3method(ggplot2::autoplot,matching_result)
S3method(ggplot2::autoplot,sensitivity_analysis)
S3method(join_matched,cem_result)
S3method(join_matched,default)
S3method(join_matched,full_matching_result)
S3method(join_matched,matching_result)
S3method(join_matched,subclass_result)
S3method(match_data,cem_result)
S3method(match_data,full_matching_result)
S3method(match_data,matching_result)
S3method(match_data,subclass_result)
S3method(plot,balance_diagnostics)
S3method(plot,matching_result)
S3method(plot,sensitivity_analysis)
S3method(print,assignment_duals_result)
S3method(print,balance_diagnostics)
S3method(print,bottleneck_result)
S3method(print,cem_result)
S3method(print,distance_object)
S3method(print,full_matching_result)
S3method(print,lap_line_metric_result)
S3method(print,lap_solve_batch_result)
S3method(print,lap_solve_kbest_result)
S3method(print,lap_solve_result)
S3method(print,matching_result)
S3method(print,matchmaker_result)
S3method(print,preprocessing_result)
S3method(print,sensitivity_analysis)
S3method(print,subclass_result)
S3method(print,summary.balance_diagnostics)
S3method(print,summary.matching_result)
S3method(print,summary.sensitivity_analysis)
S3method(print,variable_health)
S3method(summary,balance_diagnostics)
S3method(summary,distance_object)
S3method(summary,lap_solve_kbest_result)
S3method(summary,matching_result)
S3method(summary,sensitivity_analysis)
export(animated_methods)
export(as_assignment_matrix)
export(as_matchit)
export(assignment)
export(assignment_duals)
export(augment)
export(bal.tab.cem_result)
export(bal.tab.full_matching_result)
export(bal.tab.matching_result)
export(bal.tab.subclass_result)
export(balance_diagnostics)
export(balance_table)
export(bottleneck_assignment)
export(cardinality_match)
export(cem_match)
export(compute_distances)
export(diagnose_distance_matrix)
export(example_costs)
export(example_df)
export(full_match)
export(get_method_used)
export(get_total_cost)
export(group_by)
export(is_distance_object)
export(is_lap_solve_batch_result)
export(is_lap_solve_kbest_result)
export(is_lap_solve_result)
export(join_matched)
export(lap_animate)
export(lap_solve)
export(lap_solve_batch)
export(lap_solve_kbest)
export(lap_solve_line_metric)
export(match_couples)
export(match_data)
export(matchmaker)
export(pixel_morph)
export(pixel_morph_animate)
export(preprocess_matching_vars)
export(ps_match)
export(sensitivity_analysis)
export(sinkhorn)
export(sinkhorn_to_assignment)
export(subclass_match)
export(update_constraints)
importFrom(Rcpp,evalCpp)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,first)
importFrom(dplyr,group_by)
importFrom(dplyr,group_keys)
importFrom(dplyr,group_split)
importFrom(dplyr,group_vars)
importFrom(dplyr,n)
importFrom(dplyr,summarise)
importFrom(purrr,map_dfr)
importFrom(rlang,enquo)
importFrom(rlang,eval_tidy)
importFrom(rlang,quo_is_null)
importFrom(stats,dist)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(utils,head)
importFrom(utils,type.convert)
useDynLib(couplr, .registration = TRUE)
