% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_core.R
\name{print.matching_result}
\alias{print.matching_result}
\title{Print method for matching results}
\usage{
\method{print}{matching_result}(x, ...)
}
\arguments{
\item{x}{A matching_result object}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns the input object \code{x}.
}
\description{
Print method for matching results
}
