% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_messages.R
\name{use_emoji}
\alias{use_emoji}
\title{Check if emoji should be used}
\usage{
use_emoji()
}
\value{
Logical indicating whether emoji should be used.
}
\description{
Check if emoji should be used
}
\keyword{internal}
