% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/couplr-package.R
\name{group_by}
\alias{group_by}
\title{Re-export of dplyr::group_by}
\value{
See \code{\link[dplyr]{group_by}}.
}
\description{
Re-export of dplyr::group_by
}
\keyword{internal}
