% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_sensitivity.R
\name{print.sensitivity_analysis}
\alias{print.sensitivity_analysis}
\title{Print method for sensitivity analysis}
\usage{
\method{print}{sensitivity_analysis}(x, ...)
}
\arguments{
\item{x}{A sensitivity_analysis object}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns the input object \code{x}.
}
\description{
Print method for sensitivity analysis
}
