% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_messages.R
\name{couplr_success}
\alias{couplr_success}
\title{Success message with emoji}
\usage{
couplr_success(...)
}
\value{
No return value, called for side effects (issues a message).
}
\description{
Success message with emoji
}
\keyword{internal}
