% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_messages.R
\name{err_missing_vars}
\alias{err_missing_vars}
\title{Missing variables error}
\usage{
err_missing_vars(vars, dataset = "left")
}
\value{
No return value, throws an error.
}
\description{
Missing variables error
}
\keyword{internal}
