% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_constraints.R
\name{apply_all_constraints}
\alias{apply_all_constraints}
\title{Apply all constraints to cost matrix}
\usage{
apply_all_constraints(
  cost_matrix,
  left,
  right,
  vars,
  max_distance = Inf,
  calipers = NULL,
  forbidden = NULL
)
}
\value{
Modified cost matrix with all constraints applied.
}
\description{
Main entry point for applying constraints.
}
\keyword{internal}
