% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_utils.R
\name{validate_calipers}
\alias{validate_calipers}
\title{Validate calipers parameter}
\usage{
validate_calipers(calipers, vars)
}
\value{
Validated calipers (list or named numeric), or NULL if none.
}
\description{
Validate calipers parameter
}
\keyword{internal}
