% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_core.R
\name{check_full_matching}
\alias{check_full_matching}
\title{Check if full matching was achieved}
\usage{
check_full_matching(result)
}
\value{
No return value; throws error if unmatched units exist.
}
\description{
Check if full matching was achieved
}
\keyword{internal}
