% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_cem.R
\name{.auto_breaks}
\alias{.auto_breaks}
\title{Compute automatic break points for CEM binning}
\usage{
.auto_breaks(x, n_bins)
}
\arguments{
\item{x}{Combined numeric vector from left and right}

\item{n_bins}{Binning method or integer}
}
\value{
Numeric vector of break points
}
\description{
Compute automatic break points for CEM binning
}
\keyword{internal}
