% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_utils.R
\name{validate_weights}
\alias{validate_weights}
\title{Validate weights parameter}
\usage{
validate_weights(weights, vars)
}
\value{
Numeric vector of validated weights.
}
\description{
Validate weights parameter
}
\keyword{internal}
