% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_messages.R
\name{warn_parallel_unavailable}
\alias{warn_parallel_unavailable}
\title{Parallel package missing warning (reuse from matching_parallel.R)}
\usage{
warn_parallel_unavailable()
}
\value{
No return value, called for side effects (issues a warning).
}
\description{
Parallel package missing warning (reuse from matching_parallel.R)
}
\keyword{internal}
