% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_method_used}
\alias{get_method_used}
\title{Extract method used from assignment result}
\usage{
get_method_used(x)
}
\arguments{
\item{x}{An assignment result object}
}
\value{
Character string indicating method used
}
\description{
Extract method used from assignment result
}
