% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_constraints.R
\name{has_valid_pairs}
\alias{has_valid_pairs}
\title{Check if any valid pairs exist}
\usage{
has_valid_pairs(cost_matrix)
}
\value{
Logical indicating whether any valid pairs exist.
}
\description{
Check if any valid pairs exist
}
\keyword{internal}
