% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_core.R
\name{match_couples_from_distance}
\alias{match_couples_from_distance}
\title{Match from Precomputed Distance Object}
\usage{
match_couples_from_distance(
  dist_obj,
  max_distance = Inf,
  calipers = NULL,
  ignore_blocks = FALSE,
  require_full_matching = FALSE,
  method = "auto",
  strategy = "row_best",
  return_unmatched = TRUE,
  return_diagnostics = FALSE,
  check_costs = TRUE
)
}
\value{
A matching_result object with pairs, info, and optional diagnostics.
}
\description{
Internal function to handle matching when a distance_object is provided
}
\keyword{internal}
