% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_full.R
\name{print.full_matching_result}
\alias{print.full_matching_result}
\title{Print Method for Full Matching Results}
\usage{
\method{print}{full_matching_result}(x, ...)
}
\arguments{
\item{x}{A full_matching_result object}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns the input object \code{x}.
}
\description{
Print Method for Full Matching Results
}
