% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lap_solve.R
\name{print.lap_solve_result}
\alias{print.lap_solve_result}
\title{Print method for assignment results}
\usage{
\method{print}{lap_solve_result}(x, ...)
}
\arguments{
\item{x}{A \code{lap_solve_result} object.}

\item{...}{Additional arguments passed to \code{print()}. Currently ignored.}
}
\value{
Invisibly returns the input object \code{x}.
}
\description{
Nicely prints a \code{lap_solve_result} object, including the assignments,
total cost, and method used.
}
