% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lap_solve_batch.R
\name{print.lap_solve_batch_result}
\alias{print.lap_solve_batch_result}
\title{Print method for batch assignment results}
\usage{
\method{print}{lap_solve_batch_result}(x, ...)
}
\arguments{
\item{x}{A \code{lap_solve_batch_result} object.}

\item{...}{Additional arguments passed to \code{print()}. Currently ignored.}
}
\value{
Invisibly returns the input object \code{x}.
}
\description{
Prints a summary and the table of results for a batch of assignment
problems solved with \code{lap_solve_batch()}.
}
