% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_parallel.R
\name{can_parallelize}
\alias{can_parallelize}
\title{Check if parallel processing is available}
\usage{
can_parallelize()
}
\value{
Logical indicating if future package is available
}
\description{
Check if parallel processing is available
}
\keyword{internal}
