% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is_lap_solve_batch_result}
\alias{is_lap_solve_batch_result}
\title{Check if object is a batch assignment result}
\usage{
is_lap_solve_batch_result(x)
}
\arguments{
\item{x}{Object to test}
}
\value{
Logical indicating if x is a batch assignment result
}
\description{
Check if object is a batch assignment result
}
