% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_subclass.R
\name{print.subclass_result}
\alias{print.subclass_result}
\title{Print Method for Subclassification Results}
\usage{
\method{print}{subclass_result}(x, ...)
}
\arguments{
\item{x}{A subclass_result object}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns the input object \code{x}.
}
\description{
Print Method for Subclassification Results
}
