% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_blocks.R
\name{summarize_blocks}
\alias{summarize_blocks}
\title{Summarize block structure}
\usage{
summarize_blocks(left, right, block_vars = NULL)
}
\value{
Tibble with block_id, n_left, n_right, and optional variable means.
}
\description{
Summarize block structure
}
\keyword{internal}
