% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_sensitivity.R
\name{summary.sensitivity_analysis}
\alias{summary.sensitivity_analysis}
\title{Summary method for sensitivity analysis}
\usage{
\method{summary}{sensitivity_analysis}(object, ...)
}
\arguments{
\item{object}{A sensitivity_analysis object}

\item{...}{Additional arguments (ignored)}
}
\value{
A list containing summary statistics (invisibly)
}
\description{
Summary method for sensitivity analysis
}
