% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_cem.R
\name{print.cem_result}
\alias{print.cem_result}
\title{Print Method for CEM Results}
\usage{
\method{print}{cem_result}(x, ...)
}
\arguments{
\item{x}{A cem_result object}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns the input object \code{x}.
}
\description{
Print Method for CEM Results
}
