% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_sensitivity.R
\name{.rosenbaum_bounds}
\alias{.rosenbaum_bounds}
\title{Compute Rosenbaum bounds via normal approximation}
\usage{
.rosenbaum_bounds(t_obs, ranks, n, gamma, alternative)
}
\value{
List with p_upper and p_lower.
}
\description{
Under hidden bias Gamma, each pair's probability of the treated unit
having the larger value is between 1/(1+Gamma) and Gamma/(1+Gamma).
}
\keyword{internal}
