% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_cardinality.R
\name{.pair_var_diffs}
\alias{.pair_var_diffs}
\title{Compute per-pair differences on a single variable}
\usage{
.pair_var_diffs(pairs, left, right, var, left_id_col, right_id_col)
}
\description{
Compute per-pair differences on a single variable
}
\keyword{internal}
