% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_diagnostics.R
\name{summary.balance_diagnostics}
\alias{summary.balance_diagnostics}
\title{Summary method for balance diagnostics}
\usage{
\method{summary}{balance_diagnostics}(object, ...)
}
\arguments{
\item{object}{A balance_diagnostics object}

\item{...}{Additional arguments (ignored)}
}
\value{
A list containing summary statistics (invisibly)
}
\description{
Summary method for balance diagnostics
}
