% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_parallel.R
\name{parallel_lapply}
\alias{parallel_lapply}
\title{Parallel lapply using future}
\usage{
parallel_lapply(X, FUN, ..., parallel = FALSE)
}
\arguments{
\item{X}{Vector to iterate over}

\item{FUN}{Function to apply}

\item{...}{Additional arguments to FUN}

\item{parallel}{Whether parallel processing is enabled}
}
\value{
List of results
}
\description{
Parallel lapply using future
}
\keyword{internal}
