% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_constraints.R
\name{BIG_COST}
\alias{BIG_COST}
\title{Large value for forbidden pairs}
\format{
Numeric value (half of .Machine$double.xmax).
}
\usage{
BIG_COST
}
\description{
A numeric constant used to mark forbidden pairs in cost matrices.
}
\keyword{internal}
