% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/animate.R
\name{animated_methods}
\alias{animated_methods}
\title{List methods that currently support animation}
\usage{
animated_methods()
}
\value{
Character vector of registered method names.
}
\description{
Returns the character vector of method strings for which a trace function
has been registered. Use any of these with \code{\link[=lap_animate]{lap_animate()}}.
}
