% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_lazy.R
\name{new_lazy_cost_spec}
\alias{new_lazy_cost_spec}
\title{Construct a lazy cost specification}
\usage{
new_lazy_cost_spec(left_mat, right_mat, distance, sigma, weights, vars)
}
\value{
An object of class "lazy_cost_spec".
}
\description{
Construct a lazy cost specification
}
\keyword{internal}
