% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_constraints.R
\name{count_valid_pairs}
\alias{count_valid_pairs}
\title{Count valid pairs in cost matrix}
\usage{
count_valid_pairs(cost_matrix)
}
\value{
Integer count of valid (non-forbidden) pairs.
}
\description{
Count valid pairs in cost matrix
}
\keyword{internal}
