% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_total_cost}
\alias{get_total_cost}
\title{Extract total cost from assignment result}
\usage{
get_total_cost(x)
}
\arguments{
\item{x}{An assignment result object}
}
\value{
Numeric total cost
}
\description{
Extract total cost from assignment result
}
