% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is_lap_solve_result}
\alias{is_lap_solve_result}
\title{Check if object is an assignment result}
\usage{
is_lap_solve_result(x)
}
\arguments{
\item{x}{Object to test}
}
\value{
Logical indicating if x is an assignment result
}
\description{
Check if object is an assignment result
}
