% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_plots.R
\name{.autoplot_hist}
\alias{.autoplot_hist}
\title{Histogram of |std diff| via ggplot2}
\usage{
.autoplot_hist(object, threshold = 0.1)
}
\description{
Histogram of |std diff| via ggplot2
}
\keyword{internal}
