% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_messages.R
\name{check_cost_distribution}
\alias{check_cost_distribution}
\title{Check cost distribution for problems}
\usage{
check_cost_distribution(cost_matrix, threshold_zero = 1e-10, warn = TRUE)
}
\arguments{
\item{cost_matrix}{Numeric matrix of distances}

\item{threshold_zero}{Threshold for considering distance "zero" (default: 1e-10)}

\item{warn}{If TRUE, issue warnings for problems found}
}
\value{
List with diagnostic information
}
\description{
Examines the distance matrix for common issues and provides helpful warnings.
}
\keyword{internal}
