% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_messages.R
\name{couplr_stop}
\alias{couplr_stop}
\title{Stop with a fun, themed error message}
\usage{
couplr_stop(..., call. = FALSE)
}
\value{
No return value, throws an error.
}
\description{
Stop with a fun, themed error message
}
\keyword{internal}
