% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_parallel.R
\name{restore_parallel}
\alias{restore_parallel}
\title{Restore original parallel plan}
\usage{
restore_parallel(parallel_state)
}
\arguments{
\item{parallel_state}{State from setup_parallel()}
}
\value{
No return value, called for side effects (restores parallel plan).
}
\description{
Restore original parallel plan
}
\keyword{internal}
