% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_plots.R
\name{.autoplot_love}
\alias{.autoplot_love}
\title{Love plot via ggplot2}
\usage{
.autoplot_love(object, threshold = 0.1)
}
\description{
Love plot via ggplot2
}
\keyword{internal}
