% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_utils.R
\name{extract_ids}
\alias{extract_ids}
\title{Extract and standardize IDs from data frames}
\usage{
extract_ids(df, prefix = "id")
}
\value{
Character vector of IDs.
}
\description{
Extract and standardize IDs from data frames
}
\keyword{internal}
