% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_core.R
\name{.couples_replace}
\alias{.couples_replace}
\title{Replacement matching: each left picks its best right independently}
\usage{
.couples_replace(
  cost_matrix,
  left,
  right,
  left_ids,
  right_ids,
  vars,
  ratio = 1L
)
}
\value{
List with pairs tibble, unmatched list, and info list.
}
\description{
Replacement matching: each left picks its best right independently
}
\keyword{internal}
