% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_core.R
\name{match_couples_single}
\alias{match_couples_single}
\title{Match without blocking (single problem)}
\usage{
match_couples_single(
  left,
  right,
  left_ids,
  right_ids,
  vars,
  distance,
  weights,
  scale,
  max_distance,
  calipers,
  method,
  strategy = "row_best",
  check_costs = TRUE,
  replace = FALSE,
  ratio = 1L,
  sigma = NULL,
  memory_mode = "auto"
)
}
\value{
List with pairs tibble and matching info.
}
\description{
Match without blocking (single problem)
}
\keyword{internal}
