% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lap_solve_kbest.R
\name{print.lap_solve_kbest_result}
\alias{print.lap_solve_kbest_result}
\title{Print method for k-best assignment results}
\usage{
\method{print}{lap_solve_kbest_result}(x, ...)
}
\arguments{
\item{x}{A \code{lap_solve_kbest_result}.}

\item{...}{Additional arguments passed to \code{print()}. Ignored.}
}
\value{
Invisibly returns the input object \code{x}.
}
\description{
Print method for k-best assignment results
}
