% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_plots.R
\name{.autoplot_variance}
\alias{.autoplot_variance}
\title{Variance ratio plot via ggplot2}
\usage{
.autoplot_variance(object)
}
\description{
Variance ratio plot via ggplot2
}
\keyword{internal}
