% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_cardinality.R
\name{.compute_pair_balance}
\alias{.compute_pair_balance}
\title{Compute standardized differences for current pairs}
\usage{
.compute_pair_balance(pairs, left, right, vars, left_id_col, right_id_col)
}
\description{
Compute standardized differences for current pairs
}
\keyword{internal}
