% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_blocks.R
\name{print.matchmaker_result}
\alias{print.matchmaker_result}
\title{Print method for matchmaker results}
\usage{
\method{print}{matchmaker_result}(x, ...)
}
\arguments{
\item{x}{A matchmaker_result object}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns the input object \code{x}.
}
\description{
Print method for matchmaker results
}
