% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_plots.R
\name{autoplot.sensitivity_analysis}
\alias{autoplot.sensitivity_analysis}
\title{ggplot2 autoplot for sensitivity analysis}
\usage{
\method{autoplot}{sensitivity_analysis}(object, alpha = 0.05, ...)
}
\arguments{
\item{object}{A sensitivity_analysis object}

\item{alpha}{Significance level (default: 0.05)}

\item{...}{Additional arguments (ignored)}
}
\value{
A ggplot object
}
\description{
Plots p-value upper bounds against sensitivity parameter Gamma.
}
