% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_parallel.R
\name{greedy_blocks_parallel}
\alias{greedy_blocks_parallel}
\title{Greedy match blocks in parallel}
\usage{
greedy_blocks_parallel(
  blocks,
  left,
  right,
  left_ids,
  right_ids,
  block_col,
  vars,
  distance,
  weights,
  scale,
  max_distance,
  calipers,
  strategy,
  parallel = FALSE
)
}
\arguments{
\item{blocks}{Vector of block IDs}

\item{left}{Left dataset with block_col}

\item{right}{Right dataset with block_col}

\item{left_ids}{IDs from left}

\item{right_ids}{IDs from right}

\item{block_col}{Name of blocking column}

\item{vars}{Variables for matching}

\item{distance}{Distance metric}

\item{weights}{Variable weights}

\item{scale}{Scaling method}

\item{max_distance}{Maximum distance}

\item{calipers}{Caliper constraints}

\item{strategy}{Greedy strategy}

\item{parallel}{Whether to use parallel processing}
}
\value{
List with combined results from all blocks
}
\description{
Greedy match blocks in parallel
}
\keyword{internal}
