% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_parallel.R
\name{setup_parallel}
\alias{setup_parallel}
\title{Setup parallel processing with future}
\usage{
setup_parallel(parallel = FALSE, n_workers = NULL)
}
\arguments{
\item{parallel}{Logical or plan specification}

\item{n_workers}{Number of workers (NULL for auto-detect)}
}
\value{
List with original plan and whether we set up parallelization
}
\description{
Setup parallel processing with future
}
\keyword{internal}
