% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_messages.R
\name{diagnose_distance_matrix}
\alias{diagnose_distance_matrix}
\title{Diagnose distance matrix and suggest fixes}
\usage{
diagnose_distance_matrix(
  cost_matrix,
  left = NULL,
  right = NULL,
  vars = NULL,
  warn = TRUE
)
}
\arguments{
\item{cost_matrix}{Numeric matrix of distances}

\item{left}{Left dataset (for variable checking)}

\item{right}{Right dataset (for variable checking)}

\item{vars}{Variables used for matching}

\item{warn}{If TRUE, issue warnings}
}
\value{
List with diagnostic results and suggestions
}
\description{
Comprehensive diagnostics for a distance matrix with actionable suggestions.
}
