% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_messages.R
\name{couplr_inform}
\alias{couplr_inform}
\title{Info message with emoji}
\usage{
couplr_inform(...)
}
\value{
No return value, called for side effects (issues a message).
}
\description{
Info message with emoji
}
\keyword{internal}
