% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_diagnostics.R
\name{print.balance_diagnostics}
\alias{print.balance_diagnostics}
\title{Print Method for Balance Diagnostics}
\usage{
\method{print}{balance_diagnostics}(x, ...)
}
\arguments{
\item{x}{A balance_diagnostics object}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns the input object \code{x}.
}
\description{
Print Method for Balance Diagnostics
}
