% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_distance.R
\name{apply_weights}
\alias{apply_weights}
\title{Apply weights to matching variables}
\usage{
apply_weights(mat, weights)
}
\value{
Numeric matrix with columns weighted.
}
\description{
Apply weights to matching variables
}
\keyword{internal}
