% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_interop.R
\name{bal.tab.matching_result}
\alias{bal.tab.matching_result}
\alias{bal.tab.full_matching_result}
\alias{bal.tab.cem_result}
\alias{bal.tab.subclass_result}
\title{Balance Table for Matching Results (cobalt integration)}
\usage{
bal.tab.matching_result(x, left, right, ...)

bal.tab.full_matching_result(x, left, right, ...)

bal.tab.cem_result(x, left, right, ...)

bal.tab.subclass_result(x, data = NULL, ...)
}
\arguments{
\item{x}{A couplr result object}

\item{left}{Data frame of left (treated) units}

\item{right}{Data frame of right (control) units}

\item{...}{Additional arguments passed to \code{cobalt::bal.tab()}}

\item{data}{Data frame used for subclassification (for subclass_result only)}
}
\value{
A cobalt balance table object
}
\description{
S3 method enabling \code{cobalt::bal.tab()} on couplr result objects.
Requires the \pkg{cobalt} package to be installed.
}
\details{
These methods convert couplr results to the format cobalt expects
(a matchit-class object) and then delegate to cobalt's own
\code{bal.tab.matchit()} method. The \pkg{cobalt} package must be
installed but is not required for couplr to function.
}
