% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_messages.R
\name{warn_extreme_costs}
\alias{warn_extreme_costs}
\title{Extreme cost ratio warning}
\usage{
warn_extreme_costs(p95, p99, ratio, problem_vars = NULL)
}
\value{
No return value, called for side effects (issues a warning).
}
\description{
Extreme cost ratio warning
}
\keyword{internal}
