% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_messages.R
\name{err_no_valid_pairs}
\alias{err_no_valid_pairs}
\title{All pairs forbidden error}
\usage{
err_no_valid_pairs(reason = NULL)
}
\value{
No return value, throws an error.
}
\description{
All pairs forbidden error
}
\keyword{internal}
