% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_sensitivity.R
\name{plot.sensitivity_analysis}
\alias{plot.sensitivity_analysis}
\title{Plot method for sensitivity analysis (base graphics)}
\usage{
\method{plot}{sensitivity_analysis}(x, alpha = 0.05, ...)
}
\arguments{
\item{x}{A sensitivity_analysis object}

\item{alpha}{Significance level (default: 0.05)}

\item{...}{Additional arguments passed to plot}
}
\value{
The sensitivity_analysis object (invisibly)
}
\description{
Plot method for sensitivity analysis (base graphics)
}
