% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_preprocessing.R
\name{print.preprocessing_result}
\alias{print.preprocessing_result}
\title{Print method for preprocessing result}
\usage{
\method{print}{preprocessing_result}(x, ...)
}
\arguments{
\item{x}{A preprocessing_result object}

\item{...}{Additional arguments (ignored)}
}
\value{
Invisibly returns the input object \code{x}.
}
\description{
Print method for preprocessing result
}
