% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matching_messages.R
\name{couplr_emoji}
\alias{couplr_emoji}
\title{Get a themed emoji}
\usage{
couplr_emoji(
  type = c("error", "warning", "info", "success", "heart", "broken", "sparkles",
    "search", "chart", "warning_sign", "stop", "check")
)
}
\value{
Character string with the emoji (or empty string if emoji disabled).
}
\description{
Get a themed emoji
}
\keyword{internal}
